/******************************************************************************
 * @brief    λ(οlinux/kfifo)
 *
 * Copyright (c) 2016~2021, <morro_luo@163.com>
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Change Logs: 
 * Date           Author       Notes 
 * 2016-05-30     roger.luo    Initial version
 * 2021-02-05     roger.luo    ӿпռȡӿ.
 ******************************************************************************/
#include "ringbuffer.h"
#include <string.h>
#include <stddef.h>

#define min(a,b) ( (a) < (b) )? (a):(b)     
     
/*
 *@brief      һջλ
 *@param[in]  r    - λ
 *@param[in]  buf  - ݻ
 *@param[in]  len  - buf(2N)
 *@retval     bool
 */
bool ring_buf_init(ring_buf_t *r,unsigned char *buf, unsigned int len)
{
    r->buf    = buf;
    r->size   = len;
    r->front  = r->rear = 0;
    return buf != NULL && (len & len -1) == 0;
}

/*
 *@brief      ջλ 
 *@param[in]  r - յĻλ
 *@retval     none
 */
void ring_buf_clr(ring_buf_t *r)
{
    r->front = r->rear = 0;
}

/*
 *@brief      ȡλݳ
 *@retval     λЧֽ 
 */
unsigned int ring_buf_len(ring_buf_t *r)
{
    return r->rear - r->front;
}

/*
 *@brief      ȡλпռ
 *@retval     пռ
 */
unsigned int ring_buf_free_space(ring_buf_t *r)
{
    return r->size - (unsigned int)(r->rear - r->front);
}

/*
 *@brief       ָȵݷŵλ 
 *@param[in]   buf - ݻ
 *             len -  
 *@retval      ʵʷŵе 
 */
unsigned int ring_buf_put(ring_buf_t *r,unsigned char *buf,unsigned int len)
{
    unsigned int i;
    unsigned int left;
    left = r->size + r->front - r->rear;
    len  = min(len , left);
    i    = min(len, r->size - (r->rear & r->size - 1));   
    memcpy(r->buf + (r->rear & r->size - 1), buf, i); 
    memcpy(r->buf, buf + i, len - i);
    r->rear += len;     
    return len;
    
}

/*
 *@brief       ӻλжȡָȵ 
 *@param[in]   len - ȡ 
 *@param[out]  buf - ݻ
 *@retval      ʵʶȡ 
 */
unsigned int ring_buf_get(ring_buf_t *r,unsigned char *buf,unsigned int len)
{
    unsigned int i;
    unsigned int left;   
    if (ring_buf_len(r) == 0)
        return 0;
    left = r->rear - r->front;
    len  = min(len , left);                                
    i    = min(len, r->size - (r->front & r->size - 1));
    memcpy(buf, r->buf + (r->front & r->size - 1), i);    
    memcpy(buf + i, r->buf, len - i);   
    r->front += len;
    return len;
}
